unit GameEngine;

interface

uses
  SysUtils, Classes, Dialogs;

Const
       MaxX = 7 ;
       MaxY = 6 ;
       Empty = 0 ;
       LinePoints = 4;
       P1 = 1 ;
       P2 = 2 ;
       NE=1; SW=2; E=3; W=4; SE=5; NW=6; S=7;

type

  TGameGrid = Array[1..MaxX, 1..MaxY] of Integer;
  TPosition = Record
                X : Integer;
                Y : Integer;
              End;
  TWantedPositions = Array[1..LinePoints]of TPosition;


  TPlayer = Record
              ID : Integer;
              Name : String;
              Score : Integer;
            End;


  TGameEngine = class(TComponent)
  private
    { Dclarations prives }
    FGrid : TGameGrid;
    FPlayerOne : TPlayer;
    FPlayerTow : TPlayer;
    FCurrentPlayer : TPlayer;

    FRoundCount : Integer;
    FGameNullCount : Integer;
    FLastPosition : TPosition;
    FLastPlayerID : Integer;

    FWantedPositions : TWantedPositions;
    FWantedPosCount : Integer;
    
    FCurrentDirection : Integer;

    FLockGame : Boolean;

    FOnInitialisation : TNotifyEvent;
    FOnNewGame : TNotifyEvent;
    FOnPlayerChage : TNotifyEvent;
    FOnOverFlow : TNotifyEvent;
    FOnPlay : TNotifyEvent;
    FOnGameNull : TNotifyEvent;
    FOnGameWiner : TNotifyEvent;

    Procedure SetPlayerOneName(N : String);
    Procedure SetPlayerTowName(N : String);
    Procedure EmptyGrid ;
    Procedure ChangePlayer;
    Function  GetFreePostion(X: integer):TPosition;
    Procedure PutValue(P:TPosition; Value:Integer);
    Function  CompareValues(O, P:TPosition):Boolean;
    Function  EndGame : Boolean;
    Procedure ClearWantedPosition ;
    Function  NextPosition(Origine:TPosition; Direction : Integer):TPosition;
    Function  SearchForWiner(Origine: TPosition) : Boolean;
    Procedure UpdateScore;
  protected
    { Dclarations protges }
  public
    { Dclarations publiques }
    Procedure InitGame;
    Procedure NewGame;
    Function  GetRoundCount : Integer;
    Function  GetGameNullCount : Integer;
    Function  GatPlayerScore(Id : Integer):Integer;
    Function  GetCurrentPlayerID : Integer;
    Function  GetCurrentPlayerName : String;
    Procedure Play(X : integer);
    Function  GetLastPosition : TPosition;
    Function  GetLastPlayerID : Integer;
    Function  GetValue(P : TPosition): Integer;
    Function  GetSuccessPoints : TWantedPositions;
  published
    { Dclarations publies }
   Property Player1Name : String Read FPlayerOne.Name Write SetPlayerOneName;
   Property Player2Name : String Read FPlayerTow.Name Write SetPlayerTowName;
   Property OnInitialisation: TNotifyEvent Read FOnInitialisation Write FOnInitialisation ;
   Property OnNewGame: TNotifyEvent Read FOnNewGame Write FOnNewGame;
   Property OnPlayerChage: TNotifyEvent Read FOnPlayerChage Write FOnPlayerChage ;
   Property OnOverFlow: TNotifyEvent Read FOnOverFlow Write FOnOverFlow ;
   Property OnPlay: TNotifyEvent Read  FOnPlay Write FOnPlay ;
   Property OnGameNull: TNotifyEvent  Read FOnGameNull Write FOnGameNull;
   Property OnGameWiner: TNotifyEvent  Read FOnGameWiner Write FOnGameWiner ;
  end;


Const
  OutOfRange : TPosition =(X :0 ; Y:0);




procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Exemples', [TGameEngine]);
end;

procedure TGameEngine.InitGame;
  begin
    FPlayerOne.ID := P1;
    FPlayerOne.Score := 0;
    FPlayerTow.ID := P2;
    FPlayerTow.Score := 0;
    FRoundCount := 0 ;
    FGameNullCount := 0;
    FCurrentDirection := 0;
    FLastPosition := OutOfRange;
    FLockGame := False;
    FCurrentPlayer := FPlayerOne;
    EmptyGrid;
    if Assigned(FOnInitialisation) then FOnInitialisation(self);
  End;

Function TGameEngine.GetCurrentPlayerID : Integer;
  begin
      result := FCurrentPlayer.ID;
  End;

Function TGameEngine.GetCurrentPlayerName : String;
  begin
      result := FCurrentPlayer.Name;
  End;



Procedure TGameEngine.NewGame;
  Begin
    EmptyGrid;
    inc(FRoundCount);
    FLockGame := False;
    if Assigned(FOnNewGame) then FOnNewGame(self);
  End;

Function TGameEngine.GetRoundCount : Integer;
  Begin
     Result := FRoundCount;
  End;

Function  TGameEngine.GetGameNullCount : Integer;
  begin
    Result := FGameNullCount;
  End;

Function  TGameEngine.GatPlayerScore(Id : Integer):Integer;
  Begin
     Case id of
      P1 : Result := FPlayerOne.Score;
      P2 : Result := FPlayerTow.Score;
      else Result :=0;
     end
  End;

Procedure TGameEngine.SetPlayerOneName(N : String);
  Begin
    FPlayerOne.Name := N;
  End;

Procedure TGameEngine.SetPlayerTowName(N : String);
  Begin
    FPlayerTow.Name := N;
  End;

Procedure TGameEngine.EmptyGrid ;
  Var x,y : integer;
  begin
     for x:= 1 to MaxX do
     For y:= 1 to MaxY do
     FGrid[x,y]:= Empty;
  End;

Procedure TGameEngine.ChangePlayer;
  begin
    if FCurrentPlayer.id = P1 then FCurrentPlayer := FPlayerTow else FCurrentPlayer := FPlayerOne;
    if Assigned(FOnPlayerChage) then FOnPlayerChage(self);
  End;

Function TGameEngine.GetFreePostion(X: integer):TPosition;
  Var Y : integer;
      P : TPosition;
      Found : Boolean;
  Begin
     Found := False;
     P.X := X ; Y :=1 ;
     Repeat
       if FGrid[x,y]= Empty Then
         begin
          P.Y := Y;
          Found := True;
         End else
           begin
             Y:=Y+1;
           End;

     until (Found = True) or (Y > MaxY);
     if found = true then Result := P else Result := OutOfRange;

  End;

Procedure TGameEngine.PutValue(P:TPosition; Value:Integer);
  begin
     FGrid[P.X, P.Y]:= Value;
  End;

Function TGameEngine.GetValue(P : TPosition): Integer;
  begin
    Result := FGrid[P.X,P.Y];
  End;

Function  TGameEngine.GetSuccessPoints : TWantedPositions;
  Begin
    Result := FWantedPositions;
  End;

//****************************************************************************
Procedure TGameEngine.Play(X : integer);
   Var P:Tposition;
       id : integer;
   begin
    if FLockGame = True then Exit;
    id:= FCurrentPlayer.ID;
    P := GetFreePostion(X);
    if (P.X = 0)Or (P.Y=0) then
      begin
        if Assigned(FOnOverFlow) then FOnOverFlow(self);
      End else
         begin
           PutValue(P,id);
           FLastPosition := P;
           FLastPlayerID := ID;
           if Assigned(FOnPlay) then FOnPlay(self);
           if SearchForWiner(P) = True Then
             Begin
               FLockGame := True;
               if Assigned(FOnGameWiner) Then FOnGameWiner(self);
             End else
                 begin
                   if EndGame = True Then
                      begin
                        FLockGame := True;
                        inc(FGameNullCount);
                        if Assigned(FOnGameNull) Then FOnGameNull(self);
                      end else
                          begin

                          End;
                 end;
           ChangePlayer;
         End;
   End;
//****************************************************************************
Function TGameEngine.GetLastPosition : TPosition;
   Begin
     Result := FLastPosition;
   End;

Function TGameEngine.GetLastPlayerID : Integer;
   Begin
    Result := FLastPlayerID;
   End;

Function TGameEngine.CompareValues(O, P:TPosition):Boolean;
  Var vO, vP : Integer;
  Begin
    vO:= GetValue(O);  vP := GetValue(P);
    if vO = vP Then  Result := True else  Result := False;
  End;

Function TGameEngine.EndGame : Boolean;
  Var x, y : integer;
  begin
     Result := True;
     For x:= 1 to MaxX do
     For y:= 1 to MaxY do
     if FGrid[x,y]=Empty then Result := False;
  End;

Procedure TGameEngine.ClearWantedPosition ;
  var x : integer;
  Begin
    For x:= 2 to LinePoints do FWantedPositions[x]:= OutOfRange;
    FWantedPosCount := 1 ;
  End;


Function  TGameEngine.NextPosition(Origine:TPosition; Direction : Integer):TPosition;
  Var dX, dY : Integer;
      P : TPosition;
  Begin
     Case Direction of
       NE: begin dX:= 1 ; dY:= 1 ; end;
       SW: begin dX:=-1 ; dY:=-1  ; end;
       E : begin dX:= 1 ; dY:= 0 ; end;
       W : begin dX:=-1 ; dY:= 0 ; end;
       SE: begin dX:= 1 ; dY:=-1 ; end;
       NW: begin dX:=-1 ; dY:= 1 ; end;
       S : begin dX:= 0 ; dY:=-1  ; end;
       //else begin dx := MaxX; dY:= MaxY; end;
     end;// end case
     P.X := Origine.X + dX;
     P.Y := Origine.Y + dY;
     if Not(P.X In [1..MaxX]) Or Not(P.Y In [1..MaxY]) Then Result := OutOfRange
     else Result := P;
  End;

Procedure TGameEngine.UpdateScore;
  Begin
    if FCurrentPlayer.ID = P1 Then Inc(FPlayerOne.Score) Else Inc(FPlayerTow.Score);
  End;

//****************************************************************************
Function TGameEngine.SearchForWiner(Origine: TPosition) : Boolean;
  Var  P: TPosition;
       CanChangeDirection , Next: Boolean;
  Begin
     Result := False;
     CanChangeDirection := True;
     P := Origine;
     FCurrentDirection := 0;
     //ClearWantedPosition;
     FWantedPositions[1]:= Origine;
     Repeat
       if  CanChangeDirection = true then
           begin
             inc(FCurrentDirection);
             if FCurrentDirection = NE then ClearWantedPosition ;
             if FCurrentDirection = E then ClearWantedPosition ;
             if FCurrentDirection = SE then ClearWantedPosition ;
             if FCurrentDirection = S then ClearWantedPosition ;
           end;
       P := NextPosition(P,FCurrentDirection);
       if (P.X=0 )or (P.Y =0) then
         begin
           CanChangeDirection := True;
           Next := False;
           P := Origine;
         end else
             begin
               if CompareValues(Origine,P) = false then
                 begin
                  CanChangeDirection := True;
                  Next := False;
                  P := Origine;
                 end else
                     begin
                       Inc(FWantedPosCount);
                       FWantedPositions[FWantedPosCount]:= P;
                       CanChangeDirection := False;
                       Next := True;
                     end;
             end;
             
     Until  ((FCurrentDirection = S)and (Next = False)) or (FWantedPosCount = LinePoints);

     If FWantedPosCount = LinePoints then
       begin
         UpdateScore;
         Result := True;
       end;
  End;
//****************************************************************************



end.
